//
//  MFDefines.h
//  MFAuthLib
//
//  Created by MicroFocus on 4/25/17.
//  Copyright © 2017 Microfocus. All rights reserved.
//

#ifndef MFDefines_h
#define MFDefines_h

/*! @def MF_UNAVAILABLE_USE_INITIALIZER(designatedInitializer)
 @brief Provides a template implementation for init-family methods which have been marked as
 NS_UNAVILABLE. Stops the compiler from giving a warning when it's the super class'
 designated initializer, and gives callers useful feedback telling them what the
 new designated initializer is.
 @remarks Takes a SEL as a parameter instead of a string so that we get compiler warnings if the
 designated intializer's signature changes.
 @param designatedInitializer A SEL referencing the designated initializer.
 */
#define MF_UNAVAILABLE_USE_INITIALIZER(designatedInitializer) { \
NSString *reason = [NSString stringWithFormat:@"Called: %@\nDesignated Initializer:%@", \
NSStringFromSelector(_cmd), \
NSStringFromSelector(designatedInitializer)]; \
@throw [NSException exceptionWithName:@"Attempt to call unavailable initializer." \
reason:reason \
userInfo:nil]; \
}

#endif /* MFDefines_h */
