//
//  AccountConfig.m
//  TApp2
//
//  Created by Microfocus on 7/16/15.
//  Copyright (c) 2017 Microfocus. All rights reserved.
//

#import "AccountConfig.h"
#import "Constants.h"

@interface AccountConfig()

//@property (strong, nonatomic) NSMutableDictionary *oauthProps;

- (void) setOAuthConfigWithClientId:(NSString *) clientId
                       clientSecret:(NSString *) secret
                        redirectURI:(NSString *) redirectUri
                              scope:(NSString *) scope
                        accountName:(NSString *) accountName;

@end


@implementation AccountConfig

- (void) encodeWithCoder:(NSCoder *)encoder {
    [encoder encodeObject:[self getAccountName] forKey:kAccountName];
    [encoder encodeObject:[self getProviderUrl] forKey:kProviderUrl];
    [encoder encodeObject:[self getDiscoveryUrl] forKey:kDiscoveryUrl];
    [encoder encodeObject:[self getClientId] forKey:kOAuthClientId];
    [encoder encodeObject:[self getClientSecret] forKey:kOAuthClientSecret];
    [encoder encodeObject:[self getRedirectUri] forKey:kOAuthRedirectUri];
    [encoder encodeObject:[self getScope] forKey:kOAuthScope];
    [encoder encodeObject:[self getAuthzUrl] forKey:kOAuthAuthzUrl];
    [encoder encodeObject:[self getTokenUrl] forKey:kOAuthTokenUrl];
    [encoder encodeObject:[self getRevocationUrl] forKey:kOAuthRevocationUrl];
    [encoder encodeObject:[self getUserinfoUrl] forKey:kOAuthUserInfoUrl];
    [encoder encodeObject:[self getGrantUrl] forKey:kOAuthGrantUrl];
    [encoder encodeObject:[self getBrowserLogoutUrl] forKey:kBrowserLogoutUrl];
    [encoder encodeObject:[self getProviderDomain] forKey:kOAuthDomain];
    [encoder encodeObject:[self getOAuthDone] forKey:kOAuthDone];
    [encoder encodeObject:[self getTestPR1] forKey:kTestPR1];
}

- (id)initWithCoder:(NSCoder *)decoder {
    self = [super init];
    if (!self) {
        return nil;
    }

    if (!self.oauthProps) {
        self.oauthProps = [[NSMutableDictionary alloc] init];
    }
    
    [self.oauthProps setValue:[decoder decodeObjectForKey:kAccountName] forKey:kAccountName];
    [self.oauthProps setValue:[decoder decodeObjectForKey:kProviderUrl] forKey:kProviderUrl];
    [self.oauthProps setValue:[decoder decodeObjectForKey:kDiscoveryUrl] forKey:kDiscoveryUrl];
    [self.oauthProps setValue:[decoder decodeObjectForKey:kOAuthClientId] forKey:kOAuthClientId];
    [self.oauthProps setValue:[decoder decodeObjectForKey:kOAuthClientSecret] forKey:kOAuthClientSecret];
    [self.oauthProps setValue:[decoder decodeObjectForKey:kOAuthRedirectUri] forKey:kOAuthRedirectUri];
    [self.oauthProps setValue:[decoder decodeObjectForKey:kOAuthScope] forKey:kOAuthScope];
    [self.oauthProps setValue:[decoder decodeObjectForKey:kOAuthAuthzUrl] forKey:kOAuthAuthzUrl];
    [self.oauthProps setValue:[decoder decodeObjectForKey:kOAuthTokenUrl] forKey:kOAuthTokenUrl];
    [self.oauthProps setValue:[decoder decodeObjectForKey:kOAuthRevocationUrl] forKey:kOAuthRevocationUrl];
    [self.oauthProps setValue:[decoder decodeObjectForKey:kOAuthUserInfoUrl] forKey:kOAuthUserInfoUrl];
    [self.oauthProps setValue:[decoder decodeObjectForKey:kOAuthGrantUrl] forKey:kOAuthGrantUrl];
    [self.oauthProps setValue:[decoder decodeObjectForKey:kBrowserLogoutUrl] forKey:kBrowserLogoutUrl];
    [self.oauthProps setValue:[decoder decodeObjectForKey:kOAuthDomain] forKey:kOAuthDomain];
    [self.oauthProps setValue:[decoder decodeObjectForKey:kOAuthDone] forKey:kOAuthDone];
    [self.oauthProps setValue:[decoder decodeObjectForKey:kTestPR1] forKey:kTestPR1];
    
    return self;
}

- (id) initWithDiscoveryUrl:(NSString *) discoveryUrl
                  clientId:(NSString *) clientId
              clientSecret:(NSString *) secret
               redirectURI:(NSString *) redirectUri
               accountName:(NSString *) accountName
{
    return [self initWithDiscoveryUrl:discoveryUrl
                   revocationEndpoint:nil
                     browserLogoutUrl:nil
                             clientId:clientId
                         clientSecret:secret
                          redirectURI:redirectUri
                          accountName:accountName];
}

- (id) initWithDiscoveryUrl:(NSString *) discoveryUrl
         revocationEndpoint:(NSString *) revocationUrl
                   clientId:(NSString *) clientId
               clientSecret:(NSString *) secret
                redirectURI:(NSString *) redirectUri
                accountName:(NSString *) accountName
{
    return [self initWithDiscoveryUrl:discoveryUrl
                   revocationEndpoint:revocationUrl
                     browserLogoutUrl:nil
                             clientId:clientId
                         clientSecret:secret
                          redirectURI:redirectUri
                          accountName:accountName];
}

- (id) initWithDiscoveryUrl:(NSString *) discoveryUrl
         revocationEndpoint:(NSString *) revocationUrl
           browserLogoutUrl:(NSString *) browserLogoutUrl
                   clientId:(NSString *) clientId
               clientSecret:(NSString *) secret
                redirectURI:(NSString *) redirectUri
                accountName:(NSString *) accountName
{
    self = [super init];
    if (self) {
        self.oauthProps = [[NSMutableDictionary alloc] init];
        [self.oauthProps setValue:discoveryUrl forKey:kDiscoveryUrl];
        [self.oauthProps setValue:revocationUrl forKey:kOAuthRevocationUrl];
        [self.oauthProps setValue:browserLogoutUrl forKey:kBrowserLogoutUrl];
        [self setOAuthConfigWithClientId:clientId
                            clientSecret:secret
                             redirectURI:redirectUri
                                   scope:nil
                             accountName:accountName];
    }
    return self;
}

- (id) initWithProviderUrl:(NSString *) providerUrl
                       clientId:(NSString *) clientId
                   clientSecret:(NSString *) secret
                    redirectURI:(NSString *) redirectUri
                          scope:(NSString *) scope
                    accountName:(NSString *) accountName
{
    self = [super init];
    if (self) {
        self.oauthProps = [[NSMutableDictionary alloc] init];
        [self.oauthProps setValue:providerUrl forKey:kProviderUrl];
        [self setOAuthConfigWithClientId:clientId
                            clientSecret:secret
                             redirectURI:redirectUri
                                   scope:scope
                             accountName:accountName];
    }
    return self;
}

- (id) initWithAuthzEndpoint:(NSString *) authzUrl
               tokenEndpoint:(NSString *) tokenUrl
            userInfoEndpoint:(NSString *) userinfoUrl
                    clientId:(NSString *) clientId
                clientSecret:(NSString *) secret
                 redirectURI:(NSString *) redirectUri
                       scope:(NSString *) scope
                 accountName:(NSString *) accountName
{
    return [self initWithAuthzEndpoint:authzUrl
                         tokenEndpoint:tokenUrl
                    revocationEndpoint:nil
                      userInfoEndpoint:userinfoUrl
                      browserLogoutUrl:nil
                              clientId:clientId
                          clientSecret:secret
                           redirectURI:redirectUri
                                 scope:scope
                           accountName:accountName];
}

- (id) initWithAuthzEndpoint:(NSString *) authzUrl
               tokenEndpoint:(NSString *) tokenUrl
          revocationEndpoint:(NSString *) revocationUrl
            userInfoEndpoint:(NSString *) userinfoUrl
                    clientId:(NSString *) clientId
                clientSecret:(NSString *) secret
                 redirectURI:(NSString *) redirectUri
                       scope:(NSString *) scope
                 accountName:(NSString *) accountName
{
    return [self initWithAuthzEndpoint:authzUrl
                         tokenEndpoint:tokenUrl
                    revocationEndpoint:revocationUrl
                      userInfoEndpoint:userinfoUrl
                      browserLogoutUrl:nil
                              clientId:clientId
                          clientSecret:secret
                           redirectURI:redirectUri
                                 scope:scope
                           accountName:accountName];
}

- (id) initWithAuthzEndpoint:(NSString *) authzUrl
               tokenEndpoint:(NSString *) tokenUrl
          revocationEndpoint:(NSString *) revocationUrl
            userInfoEndpoint:(NSString *) userinfoUrl
            browserLogoutUrl:(NSString *) browserLogoutUrl
                    clientId:(NSString *) clientId
                clientSecret:(NSString *) secret
                 redirectURI:(NSString *) redirectUri
                       scope:(NSString *) scope
                 accountName:(NSString *) accountName
{
    self = [super init];
    if (self) {
        self.oauthProps = [[NSMutableDictionary alloc] init];
        [self.oauthProps setValue:authzUrl forKey:kOAuthAuthzUrl];
        [self.oauthProps setValue:tokenUrl forKey:kOAuthTokenUrl];
        [self.oauthProps setValue:revocationUrl forKey:kOAuthRevocationUrl];
        [self.oauthProps setValue:userinfoUrl forKey:kOAuthUserInfoUrl];
        [self.oauthProps setValue:browserLogoutUrl forKey:kBrowserLogoutUrl];
        [self setOAuthConfigWithClientId:clientId
                            clientSecret:secret
                             redirectURI:redirectUri
                                   scope:scope
                             accountName:accountName
         ];
    }
    return self;
}


- (id) initWithGrantEndpoint:(NSString *) grantUrl
                    clientId:(NSString *) clientId
                clientSecret:(NSString *) secret
                 redirectURI:(NSString *) redirectUri
                       scope:(NSString *) scope
                 accountName:(NSString *) accountName
{
    self = [super init];
    if (self) {
        self.oauthProps = [[NSMutableDictionary alloc] init];
        [self.oauthProps setValue:grantUrl forKey:kOAuthGrantUrl];
        [self setOAuthConfigWithClientId:clientId
                            clientSecret:secret
                             redirectURI:redirectUri
                                   scope:scope
                             accountName:accountName
         ];
    }
    return self;
}

- (void) setOAuthConfigWithClientId:(NSString *) clientId
                       clientSecret:(NSString *) secret
                        redirectURI:(NSString *) redirectUri
                              scope:(NSString *) scope
                        accountName:(NSString *) accountName
{
    [self.oauthProps setValue:clientId forKey:kOAuthClientId];
    [self.oauthProps setValue:secret forKey:kOAuthClientSecret];
    [self.oauthProps setValue:redirectUri forKey:kOAuthRedirectUri];
    [self.oauthProps setValue:scope forKey:kOAuthScope];
    [self.oauthProps setValue:accountName forKey:kAccountName];
    [self.oauthProps setValue:@"NO" forKey:kOAuthDone];
}

- (NSString *) getAccountName
{
    return [self.oauthProps objectForKey:kAccountName];
}

- (NSString *) getProviderUrl
{
    return [self.oauthProps objectForKey:kProviderUrl];
}

- (NSString *) getDiscoveryUrl
{
    return [self.oauthProps objectForKey:kDiscoveryUrl];
}

- (NSString *) getAuthzUrl
{
    return [self.oauthProps objectForKey:kOAuthAuthzUrl];
}

- (NSString *) getRevocationUrl
{
    return [self.oauthProps objectForKey:kOAuthRevocationUrl];
}

- (NSString *) getbrowserLogoutUrl
{
    return [self.oauthProps objectForKey:kBrowserLogoutUrl];
}

- (NSString *) getProviderDomain
{
    NSString *domain = nil;
    if ([[self getProviderUrl] length] > 0) {
        domain = [NSURL URLWithString:[self getProviderUrl]].host;
    } else if ([[self getAuthzUrl] length] > 0) {
        domain = [NSURL URLWithString:[self getAuthzUrl]].host;
    } else if ([[self getGrantUrl] length] > 0) {
        domain = [NSURL URLWithString:[self getGrantUrl]].host;
    }
    return domain;
}

- (NSString *) getTokenUrl
{
    return [self.oauthProps objectForKey:kOAuthTokenUrl];
}

- (NSString *) getGrantUrl
{
    return [self.oauthProps objectForKey:kOAuthGrantUrl];
}

- (NSString *) getUserinfoUrl
{
    return [self.oauthProps objectForKey:kOAuthUserInfoUrl];
}

- (NSString *) getClientId
{
    return [self.oauthProps objectForKey:kOAuthClientId];
}

- (NSString *) getClientSecret
{
    return [self.oauthProps objectForKey:kOAuthClientSecret];
}

- (NSString *) getScope
{
    return [self.oauthProps objectForKey:kOAuthScope];
}

- (NSString *) getRedirectUri
{
    return [self.oauthProps objectForKey:kOAuthRedirectUri];
}

-(NSString *) getBrowserLogoutUrl
{
    return [self.oauthProps objectForKey:kBrowserLogoutUrl];
}

- (NSString *) getOAuthDone {
    return [self.oauthProps objectForKey:kOAuthDone];
}

- (void) setOAuthDone:(NSString *) oauthDone {
    [self.oauthProps setValue:oauthDone forKey:kOAuthDone];
}

- (BOOL) isOAuthDone {
    NSString *sOAuthDone = [self getOAuthDone];
    if ([sOAuthDone caseInsensitiveCompare:@"YES"]  == NSOrderedSame)
        return YES;
    else
        return NO;
}

- (NSString *) getTestPR1 {
    return [self.oauthProps objectForKey:kTestPR1];
}

- (void) setTestPR1:(NSString *)testpr {
    [self.oauthProps setValue:testpr forKey:kTestPR1];
}

- (NSString *) description {
    return [NSString stringWithFormat:@"AccountConfig: %@", self.oauthProps];
}

- (Boolean) saveAccountConfig {
    Boolean status = [NSKeyedArchiver archiveRootObject:self toFile:@"/account.dat"];
    return status;
}

+ (AccountConfig *) loadAccountConfig {
    AccountConfig *acctConfig = [NSKeyedUnarchiver unarchiveObjectWithFile:@"/account.dat"];
    return acctConfig;
}

- (Boolean) removeAccountConfig {
    NSError *err;
    NSFileManager *fileManager = [NSFileManager defaultManager];
    BOOL success = [fileManager removeItemAtPath:@"/account.dat" error:&err];
    
    return success;
}

@end
